#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/Xutil.h>

#include "matrix.h"
#include "view.h"
#include "display.h"
#include "object.h"
#include "main.h"

#define FRUSTRUM        55.0
#define FRONT           500.0
#define DEPTH           1000.0
#define BACK            -3000.0

void            main(argc, argv)
int             argc;
char           *argv[];
{
   double          theta, elev, forw;
   unsigned int    mask;
   double          xs, ys, zs;
   int             xx, yy;

   Object         *object;
   Transformation  View;
   double          starting_point[3];
   Viewer          viewer;

   if (argc < 4)
   {
      fprintf(stderr, "Usage: %s datafile x_scale y z\n", argv[0]);
      exit(1);
   }

   viewer.initialized = FALSE;
   init_display(&viewer, 100, 100, VR, VB);      /* initialize client */

   sscanf(argv[2], "%lf", &xs);
   sscanf(argv[3], "%lf", &ys);
   sscanf(argv[4], "%lf", &zs);

   object = init_object(argv[1], xs, ys, zs);

   starting_point[0] = (object->Min_point.x + object->Max_point.x) / 2.0;
   starting_point[1] = object-> Max_point.y + 10;
   starting_point[2] = (object->Min_point.z + object->Max_point.z) / 2.0;

   forw = elev = theta = 0.0;
   update_view_info(&View, theta, elev, forw, starting_point);
   eval_projection(&View, DEPTH, FRONT, BACK, FRUSTRUM);
   eval_viewport(&View, VVL, VVR, VVT, VVB, 0.0, 1.0);
   compute_transformation(&View);

   transform_object(&View, object);
   draw_object(&viewer, &View, object);

   printf("theta= %f,forw= %f,elev= %f\n", theta, forw, elev);
   print_vrp(&View);

   while (1)
   {
      if (get_pointer(&viewer, &xx, &yy, &mask))
      {
         if ((mask == Button1Mask) || (mask == Button2Mask)
             || (mask == Button3Mask))
         {
            forw = elev = 0.0;
            switch (mask)
            {
               case Button1Mask:
                  theta -= (double) (xx - (VR / 2)) / 60.0;
                  if (theta > 360.0)
                     theta -= 360.0;
                  if (theta < 0.0)
                     theta += 360.0;
#if 0
                  forw = (double) (yy - (VB / 2)) / 60.0;
#endif
                  update_view_info(&View, theta, elev, forw, NULL);
                  break;

               case Button2Mask:
                  elev = -(double) (yy - (VB / 2)) / 60.0;
                  update_view_info(&View, theta, elev, forw, NULL);
                  break;

               case Button3Mask:
                  theta = 0.0;
                  update_view_info(&View, theta, elev, forw, starting_point);
                  break;
            }
            compute_transformation(&View);
            transform_object(&View, object);
            draw_object(&viewer, &View, object);

            printf("theta= %f,forw= %f,elev= %f\n", theta, forw, elev);
            print_vrp(&View);
         }
      }
   }
   close_display();             /* close client */
}
